package BPpackage;
 
/*
 * AudioService.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * R. Earle Harris (evsiz@this-voice.org) copyrighted 2014
 *
 */


import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.media.MediaPlayer.OnPreparedListener;
import android.os.Binder;
import android.os.IBinder;
import android.support.v4.app.NotificationCompat;
import java.io.File;
import java.io.IOException;


public class AudioService extends Service implements 
			MediaPlayer.OnPreparedListener, MediaPlayer.OnCompletionListener {

	private final IBinder mBinder = new LocalBinder();
	private NotificationManager mNMgr;
	private MediaPlayer mMediaPlayer = null;
	public PRAFileHandler fhandler = new PRAFileHandler(this);
	public String[] playlist = { "" };
	public int pos = 0;
	public boolean stopped = true;
	public boolean random = false;

	@Override
	public void onCreate() {
		super.onCreate();
		mNMgr = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
		Notification note = makeNote("");
		startForeground(1, note);
	}



	private void player(){
		if (stopped){ return; }
		mMediaPlayer = new MediaPlayer();
		mMediaPlayer.setOnPreparedListener(this);
		mMediaPlayer.setOnCompletionListener(this);
		mMediaPlayer.setAudioStreamType(AudioManager.STREAM_MUSIC);
		mMediaPlayer.setVolume(1.0f,1.0f);
		String path = playlist[pos];
		File pathf = new File(path);
		String fname = pathf.getName();
		notifier(fname);
		if (!random){
			String dname = pathf.getParent();
			String line = dname+"\t"+fname;
			fhandler.writeUniqLineByToken0("bookmarks.txt", line, "\t");
		}
		try {
			mMediaPlayer.setDataSource(path);
			mMediaPlayer.prepareAsync();
		} catch (IOException e){ ; }
	}


	public void onCompletion(MediaPlayer mp) {
		mMediaPlayer.stop();
		mMediaPlayer.release();
		mMediaPlayer = null;
		next();
	}


	public void onPrepared(MediaPlayer player) {
        player.start();
    }


    @Override
    public IBinder onBind(Intent intent) {
		return mBinder;
    }

	public void start(boolean b){
		random = b;
		if (mMediaPlayer == null){
			stopped = false;
			pos = 0;
			playlist = fhandler.getFileAsArray("playlist.txt");
			player();
		}
	}

	public void stop(){
		stopped = true;
		if (mMediaPlayer != null){
			mMediaPlayer.stop();
			mMediaPlayer.release();
			mMediaPlayer = null;
		}
	}

	public void pause(){
		if (mMediaPlayer != null){
			mMediaPlayer.pause();
		}
	}

	public void resume(){
		if (mMediaPlayer != null){
			mMediaPlayer.start();
		}
	}

	public void next(){
		if (mMediaPlayer != null){
			mMediaPlayer.stop();
			mMediaPlayer.release();
			mMediaPlayer = null;
		}
		pos++;
		if (pos >= playlist.length){
			pos = 0;
		} else if (pos < 0){
			pos = playlist.length - 1;
		} 
		player();
	}



	public void prev(){
		if (mMediaPlayer != null){
			pos = pos - 2;
			next();
		}
	}

	public class LocalBinder extends Binder {
        // Return this instance of LocalService so clients can call public methods
        AudioService getService() {
            return AudioService.this;
        }
    }


	public void notifier(String msg){
		Notification note = makeNote(msg);
		mNMgr.notify(1, note);
	 }


	private Notification makeNote(String msg){
		Intent notificationIntent = new Intent(this, Player.class);
		PendingIntent mPendingIntent = PendingIntent.getActivity(this, 0, notificationIntent, 0);
		NotificationCompat.Builder notebld = new NotificationCompat.Builder(this)
			.setContentTitle("Simplest")
			.setContentText(msg)
			.setContentIntent(mPendingIntent)
			.setSmallIcon(R.drawable.icon);
		Notification note = notebld.build();
		note.flags |= Notification.FLAG_ONGOING_EVENT | Notification.FLAG_NO_CLEAR;			
		return note;
	}

    @Override
    public void onDestroy() {
        super.onDestroy();
		mNMgr.cancelAll();
    }
}
