package BPpackage;
 
/*
 * ChooserActivity.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * R. Earle Harris (evsiz@this-voice.org) copyrighted 2014
 *
 */



import android.app.Activity;
import android.app.AlertDialog.Builder;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import it.sauronsoftware.ftp4j.FTPClient;
import it.sauronsoftware.ftp4j.FTPFile;
import java.io.File;
import java.lang.Runnable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ChooserActivity extends ListActivity {
	public final int CHS = 0;
	public final int DEL = 1;
	public final int DLD = 2;

	public Map<String, String> dbs = new HashMap<String, String>();
	static final String[] ACTIONS = new String[] { "Choose Path", "Download", "Delete" };

	public int mode = 0; // CHS by default
	public boolean exit = false;
	public File download_dir = 
		Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS);

	String user = "";
	String pswd = "";
	String srvr = "";
	String port = "";
	String path = "";
	String result = "";
	FTPClient client = new FTPClient();
	FTPFile[] flist = null;
	Item[] fileList;
	ListAdapter adapter;
	Intent intent = new Intent();
	private static final int DIALOG_LOAD_FILE = 1000;
	private Item chosenFile;
	// Stores names of traversed directories
	ArrayList<String> str = new ArrayList<String>();

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Intent origin = this.getIntent();
		user = origin.getStringExtra("USER");
		pswd = origin.getStringExtra("PSWD");
		srvr = origin.getStringExtra("SRVR");
		port = origin.getStringExtra("PORT");
		path = origin.getStringExtra("PATH");
		mode = origin.getIntExtra("MODE", mode);
		setContentView(R.layout.activity_chooser);

		result = validate();
		if (exit) {
			intent.putExtra("RESULT_STRING", result);
			setResult(RESULT_CANCELED, intent);
			finish();
		}

	}

	@Override
	public void onResume() {
		super.onResume();

		final String[] list = buildList();
		setListAdapter(new ArrayAdapter<String>(this,
				R.layout.pra_list_item, list));

		EdgeEffectListView listView = (EdgeEffectListView) getListView();
		listView.setTextFilterEnabled(true);

		listView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
				String result = ((TextView) view).getText().toString();
				String choice = dbs.get(result);
				if (choice.startsWith("choose")){
					mode = CHS;
				} else if (choice.matches("download")){
					mode = DLD;
				} else if (choice.matches("delete")){
					mode = DEL;
				}
				loadFileList();
				showDialog(DIALOG_LOAD_FILE);
				intent.putExtra("RESULT_STRING", path);
			}
		});
	}

	private String[] buildList() {
		List<String> alist = new ArrayList<String>();
		for (String f : ACTIONS) {
		    alist.add(f);
		    dbs.put(f, f.toLowerCase());
		}

		Collections.sort(alist);
		String[] list = new String[alist.size()];
		alist.toArray(list);
		return list;
	}


	private class Item {
		public FTPFile file;
		public String name;
		public int icon;
		public int what;

		public Item(FTPFile file, Integer icon) {
			this.file = file;
			if (file == null) {
				if (icon == R.drawable.icon_directory_up) {
					this.name = "Up";
					this.what = -1;
				} else {
					this.name = "Choose this dir";
					this.what = -2;
				}
			} else {
				this.name = file.getName();
				this.what = file.getType(); // file = 0; dir = 1
			}
			this.icon = icon;
		}

		@Override
		public String toString() {
			return name;
		}
	}

	private void loadFileList() {

		try {
			flist = client.list();
		} catch (Exception e) {
			return;
		}

		fileList = new Item[flist.length + 1];
		fileList[0] = new Item(null, R.drawable.icon_directory_up);
		for (int i = 0; i < flist.length; i++) {
			fileList[i + 1] = new Item(flist[i], R.drawable.icon_file);
			// Set drawables
			if (flist[i].getType() == 1) {
				fileList[i + 1].icon = R.drawable.icon_directory;
			}
		}

		String dir = "";
		try {
			dir = client.currentDirectory();
		} catch (Exception e) {
			return;
		}

		// CHS: add choose bar for directory
		if ((mode == CHS) && (!dir.matches("/"))) {
			Item temp[] = new Item[fileList.length + 1];
			temp[0] = fileList[0];
			for (int i = 1; i < fileList.length; i++) {
				temp[i + 1] = fileList[i];
			}
			temp[1] = new Item(null, R.drawable.icon_choose);
			fileList = temp;
		}

		adapter = new ArrayAdapter<Item>(this,
				android.R.layout.select_dialog_item, android.R.id.text1,
				fileList) {
			@Override
			public View getView(int position, View convertView, ViewGroup parent) {
				// creates view
				View view = super.getView(position, convertView, parent);
				TextView textView = (TextView) view
						.findViewById(android.R.id.text1);

				// put the image on the text view
				textView.setCompoundDrawablesWithIntrinsicBounds(
						fileList[position].icon, 0, 0, 0);

				// add margin between image and text (support various screen densities)
				int dp5 = (int) (5 * getResources().getDisplayMetrics().density + 0.5f);
				textView.setCompoundDrawablePadding(dp5);

				return view;
			}
		};

	}

	@Override
	protected Dialog onCreateDialog(int id) {
		Dialog dialog = null;
		AlertDialog.Builder builder = new Builder(this);

		if (fileList == null) {
			dialog = builder.create();
			return dialog;
		}

		switch (id) {
		case DIALOG_LOAD_FILE:
			if (mode == CHS) {
				builder.setTitle("Path Chooser");
			} else {
				builder.setTitle("File Chooser");
			}
			builder.setAdapter(adapter, new DialogInterface.OnClickListener() {
				@Override
				public void onClick(DialogInterface dialog, int which) {
					chosenFile = fileList[which];
					
					if (chosenFile.what == 1) {// directory
						String dirname = chosenFile.name;
						// Adds chosen directory to list
						str.add(dirname);
						fileList = null;
						try {
							client.changeDirectory(dirname);
							path = client.currentDirectory();
						} catch (Exception e) {
							return;
						}

						loadFileList();

						removeDialog(DIALOG_LOAD_FILE);
						showDialog(DIALOG_LOAD_FILE);

					} else if (chosenFile.what == -1) {// directory up

						fileList = null;
						try {
							client.changeDirectory("..");
						} catch (Exception e) {
							return;
						}
						loadFileList();

						removeDialog(DIALOG_LOAD_FILE);
						showDialog(DIALOG_LOAD_FILE);

					} else if (chosenFile.what == 0) {// file
						if (mode == CHS) {
							removeDialog(DIALOG_LOAD_FILE);
							showDialog(DIALOG_LOAD_FILE);
						} else {
						    String msg = "";
							if (mode == DEL) {
								try {
									client.deleteFile(chosenFile.name);
								} catch (Exception e) {
									exit = true;
								}
								msg = "Deleted ";
							} else if (mode == DLD){
								try {
									client.download(chosenFile.name, new File(
										   download_dir + "/" + chosenFile.name));
								} catch (Exception e) {
									exit = true;
								}
								msg = "Downloaded ";
							}
							if (!exit){
								intent.putExtra("RESULT_STRING",  msg + chosenFile.name);
								setResult(RESULT_OK, intent);
							} else {
								intent.putExtra("RESULT_STRING", "FTP error:" + chosenFile.name);
								setResult(RESULT_CANCELED, intent);
							}
							finish();
						}
						
					} else {// CHS only, choose this dir

						String dir = "";
						try {
							dir = client.currentDirectory();
						} catch (Exception e) {
							exit = true;
						}
						intent.putExtra("RESULT_STRING", dir);
						if (!exit)
							setResult(RESULT_OK, intent);
						else {
							intent.putExtra("RESULT_STRING", "Cancelled");
							setResult(RESULT_CANCELED, intent);
						}
						finish();
					}

				}
			});
			break;
		}
		dialog = builder.show();
		dialog.setOnCancelListener(new DialogInterface.OnCancelListener (){
				@Override
				public void onCancel(DialogInterface dialog){
					onBackPressed();
				}
			});		
		return dialog;
	}

	private String validate() {
		String msg = "";

		ConnectivityManager cm = (ConnectivityManager) getBaseContext()
				.getSystemService(Context.CONNECTIVITY_SERVICE);

		NetworkInfo i = cm.getActiveNetworkInfo();
		if ((i == null) || (!i.isConnected())) {
			msg = "No connection to Internet";
			exit = true;
		}

		if (!exit) {
			try { // connect
				client.connect(srvr, Integer.parseInt(port));
			} catch (Exception e) {
				e.printStackTrace();
				msg = "Failed connect with " + srvr;
				exit = true;
			}
		}

		if (!exit) {
			try { // login
				client.login(user, pswd);
			} catch (Exception e) {
				msg = "Failed login as " + user;
				exit = true;
			}
		}

		if (!exit && !path.matches("/")) {
			try { // cd to /path
				client.changeDirectory(path);
			} catch (Exception e) {
				msg = "Failed cd into " + path;
				exit = true;
			}
		}

		if (!exit) {
			try {
				flist = client.list();
				msg = client.currentDirectory();
			} catch (Exception e) {
				msg = "Server will not list directory";
				exit = true;
			}
		}
		return msg;
	}

	@Override
	public void onBackPressed() {
		intent.putExtra("RESULT_STRING", "Cancelled");
		setResult(RESULT_CANCELED, intent);
		this.finish();
	}

}
