/**
 * Android File Chooser
 * Copyright (C) 2012  ScR4tCh
 * Contact scr4tch@scr4tch.org
 *
 * This is a fork of android-file-dialog (com.lamerman) licensed new BSD
 * Take a look at https://code.google.com/p/android-file-dialog/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Lesser
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

package BPpackage;
 
/*
 * FileChooserDialog.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * R. Earle Harris (evsiz@this-voice.org) copyrighted 2014
 *
 */


import android.app.Dialog;
import android.app.ListActivity;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.text.InputType;
import android.view.KeyEvent;
import android.view.View.OnClickListener;
import android.view.View;
import android.view.Window;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;


public class FileChooserDialog extends ListActivity {

	protected Bitmap folderDrawable;
	protected Bitmap fileDrawable;
	protected Bitmap upDrawable;

	public static final int SELECT_FOLDER = 2;
	public static final int SELECT_FILE = 4;
	public static final int MODE_CREATE = 0;
	public static final int MODE_OPEN = 1;

	public static final String START_PATH = "START_PATH";
	public static final String FORMAT_FILTER = "FORMAT_FILTER";
	public static final String SELECTIONS = "FILES";
	public static final String OPERATION_MODE = "OPERATION_MODE";
	public static final String SELECT_MODE = "SELECT_MODE";
	public static final String VIEW_MODE = "VIEW_MODE";
	public static final String BROWSE_ENABLED = "BROWSE_ENABLED";
	public static final String FILE_DRAWABLE = "FILE_DRAWABLE";
	public static final String FOLDER_DRAWABLE = "FOLDER_DRAWABLE";
	public static final String UP_DRAWABLE = "UP_DRAWABLE";

	private TextView myPath;
	private EditText mFileName;
	private Button selectButton;
	private LinearLayout layoutSelect;
	private LinearLayout layoutCreate;

	private ArrayList<String> selectedFiles = new ArrayList();
	private FileChooserDialogAdapter fileAdapter;
	private FileFilter ff;
	private InputMethodManager inputManager;

	private String startPath;
	private boolean browseEnabled = true;
	private int selectMode = SELECT_FOLDER | SELECT_FILE;
	private int viewMode = SELECT_FOLDER | SELECT_FILE;
	private int operationMode = MODE_OPEN;
	private boolean selecting = false;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		requestWindowFeature(Window.FEATURE_NO_TITLE);
		folderDrawable = BitmapFactory.decodeResource(getResources(),
				R.drawable.icon_directory);
		fileDrawable = BitmapFactory.decodeResource(getResources(),
				R.drawable.icon_file);
		upDrawable = BitmapFactory.decodeResource(getResources(),
				R.drawable.icon_directory_up);
		setResult(RESULT_CANCELED, getIntent());
		setContentView(R.layout.file_dialog_main);
		myPath = (TextView) findViewById(R.id.path);
		mFileName = (EditText) findViewById(R.id.fdEditTextFile);
		inputManager = (InputMethodManager) getSystemService(INPUT_METHOD_SERVICE);

		// Select Button
		selectButton = (Button) findViewById(R.id.fdButtonSelect);
		selectButton.setText("Cancel");
		selecting = false;
		selectButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				if ((selecting) && (selectedFiles.size() != 0)) {
					getIntent().putExtra(SELECTIONS, selectedFiles);
					setResult(RESULT_OK, getIntent());
					finish();
				} else {
					finish();
				}
			}
		});

		// New Button
		final Button newButton = (Button) findViewById(R.id.fdButtonNew);
		newButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				setCreateVisible(v);

				mFileName.setText("");
				mFileName.requestFocus();
			}
		});


		// Incoming Intent
		Intent i = getIntent();
		selectMode = i.getIntExtra(SELECT_MODE, SELECT_FOLDER | SELECT_FILE);
		viewMode = i.getIntExtra(VIEW_MODE, SELECT_FOLDER | SELECT_FILE);
		browseEnabled = i.getBooleanExtra(BROWSE_ENABLED, browseEnabled);
		operationMode = i.getIntExtra(OPERATION_MODE, MODE_OPEN);
		if (operationMode == MODE_OPEN) {
			newButton.setVisibility(View.GONE);
		}

		if (i.getParcelableExtra(FORMAT_FILTER) instanceof FileFilter)
			ff = (FileFilter) i.getParcelableExtra(FORMAT_FILTER);
		if (i.getParcelableExtra(FOLDER_DRAWABLE) instanceof Bitmap)
			folderDrawable = (Bitmap) (i.getParcelableExtra(FOLDER_DRAWABLE));
		if (i.getParcelableExtra(FILE_DRAWABLE) instanceof Bitmap)
			fileDrawable = (Bitmap) (i.getParcelableExtra(FILE_DRAWABLE));
		if (i.getParcelableExtra(UP_DRAWABLE) instanceof Bitmap)
			upDrawable = (Bitmap) (i.getParcelableExtra(UP_DRAWABLE));
		// END Incoming Intent


		layoutSelect = (LinearLayout) findViewById(R.id.fdLinearLayoutSelect);
		layoutCreate = (LinearLayout) findViewById(R.id.fdLinearLayoutCreate);
		layoutCreate.setVisibility(View.GONE);

		// Cancel button
		final Button cancelButton = (Button) findViewById(R.id.fdButtonCancel);
		cancelButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				setSelectVisible(v);
			}

		});

		// Create button
		final Button createButton = (Button) findViewById(R.id.fdButtonCreate);
		createButton.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				if (mFileName.getText().length() > 0) {
					getIntent().putExtra(
							SELECTIONS,
							selectedFiles.add(fileAdapter.getCurrentPath() + File.separator
											  + mFileName.getText()));
					setResult(RESULT_OK, getIntent());
					finish();
				}
			}
		});

		EdgeEffectListView listview = (EdgeEffectListView) getListView();
		listview.setOnItemLongClickListener(new OnItemLongClickListener() {
		 
		  @Override 
		  public boolean onItemLongClick(AdapterView parentView, View view, 
										 int position, long id) { 

				File file = (File) fileAdapter.getItem(position);

				if (file == null)
					return false;

				setSelectVisible(view);
				String fname = file.getAbsolutePath();
				if (selectedFiles.contains(fname)){
					selectedFiles.remove(fname);
				} else {
					selectedFiles.add(fname);
				}
				fileAdapter.setSelected(position);
				if (selectedFiles.size() != 0){
					selectButton.setText("Select");
					selecting = true;
				}
				return true;
			}
		});



		startPath = getIntent().getStringExtra(START_PATH);
		startPath = startPath != null ? startPath : FileChooserDialogAdapter.ROOT;
		fileAdapter = new FileChooserDialogAdapter(this, startPath, viewMode, browseEnabled, ff);
		getListView().setAdapter(fileAdapter);
		myPath.setText(getText(R.string.location) + ": " + fileAdapter.getCurrentPath());

	}// END onCreate



	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		File file = (File) fileAdapter.getItem(position);

		if (file == null)
			return;

		setSelectVisible(v);

		//IF is directory
		if (file.isDirectory()) {
			selectButton.setText("Cancel");
			selecting = false;
			if (file.canRead()) {
				fileAdapter.putLastPositions(fileAdapter.getCurrentPath(),
						position);
				fileAdapter.update(file.getAbsolutePath());

				myPath.post(new Runnable() {
					public void run() {
						myPath.setText(getText(R.string.location) + ": "
								+ fileAdapter.getCurrentPath());
					}
				});

				if ((selectMode & SELECT_FOLDER) == SELECT_FOLDER) {
					String fname = file.getAbsolutePath();
					if (selectedFiles.contains(fname)){
							selectedFiles.remove(fname);
					} else {
						selectedFiles.add(fname);
					}
					fileAdapter.setSelected(position);
					if (selectedFiles.size() != 0){
						selectButton.setText("Select");
						selecting = true;
					}
				} else {
					selectedFiles.clear();
				}
			} else {

				final Dialog dialog = new Dialog(this);
				dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
				dialog.setContentView(R.layout.alert_note);
				TextView t = (TextView) dialog.findViewById(R.id.dTitle);
				t.setText(file.getName());
				TextView m = (TextView) dialog.findViewById(R.id.dNote);
				m.setText(getText(R.string.folder_unreadable));
				Button ok = (Button) dialog.findViewById(R.id.dOK);
				ok.setText("Okay");
				ok.setOnClickListener(new OnClickListener() {
						@Override
							public void onClick(View v) {
							dialog.dismiss();
						}
					});
		
				dialog.show();
			}
		//ELSE is file	
		} else {
			String fname = file.getAbsolutePath();
			if (selectedFiles.contains(fname)){
				selectedFiles.remove(fname);
			} else {
				selectedFiles.add(fname);
			}
			fileAdapter.setSelected(position);
			if (selectedFiles.size() != 0){
				selectButton.setText("Select");
				selecting = true;
			}
		}

	}// END onListItemClick





	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {

		if ((keyCode == KeyEvent.KEYCODE_BACK)) {
			selectButton.setText("Cancel");
			selecting = false;

			if (layoutCreate.getVisibility() == View.VISIBLE) {
				layoutCreate.setVisibility(View.GONE);
				layoutSelect.setVisibility(View.VISIBLE);
			} else {
				if (!fileAdapter.getCurrentPath().equals(startPath)) {
					fileAdapter.updateToParentPath();
				} else {
					return super.onKeyDown(keyCode, event);
				}
			}

			return true;
		} else {
			return super.onKeyDown(keyCode, event);
		}

	}// END onKeyDown



	private void setCreateVisible(View v) {
		layoutCreate.setVisibility(View.VISIBLE);
		layoutSelect.setVisibility(View.GONE);

		inputManager.hideSoftInputFromWindow(v.getWindowToken(), 0);
		selectButton.setText("Cancel");
		selecting = false;
	}



	private void setSelectVisible(View v) {
		layoutCreate.setVisibility(View.GONE);
		layoutSelect.setVisibility(View.VISIBLE);

		inputManager.hideSoftInputFromWindow(v.getWindowToken(), 0);
		selectButton.setText("Cancel");
		selecting = false;
	}

}
