package BPpackage;
 
/*
 * PRAWebView.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * R. Earle Harris (evsiz@this-voice.org) copyrighted 2014
 *
 */


import android.content.Context;
import android.util.AttributeSet;
import android.view.GestureDetector;
import android.view.Gravity;
import android.view.MotionEvent;
import android.webkit.WebView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.List;

public class PRAWebView extends WebView implements
		GestureDetector.OnDoubleTapListener {

	public Context mContext;
	public PRAFileHandler fhandler;
	public String sep = "\t"; // must be invalid URL char
	public String[] contextList;
	public boolean contextTravel = false;
	public final PRAWebView mWV = this;
	public int contextLength;
	public int current = 0;
	public BoilerPlate Master;

	// deprecated usage used to let WebView handle own gestures
	@SuppressWarnings("deprecation")
	public GestureDetector gestureDetector = new GestureDetector(
			new mGestureDetector());

	public PRAWebView(Context context) {
		super(context);
		mContext = context;
		fhandler = new PRAFileHandler(mContext);
		gestureDetector.setOnDoubleTapListener(this);
		gestureDetector.setIsLongpressEnabled(true);
	}

	public PRAWebView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		mContext = context;
		fhandler = new PRAFileHandler(mContext);
		gestureDetector.setOnDoubleTapListener(this);
		gestureDetector.setIsLongpressEnabled(true);
	}

	public PRAWebView(Context context, AttributeSet attrs) {
		super(context, attrs);
		mContext = context;
		fhandler = new PRAFileHandler(mContext);
		gestureDetector.setOnDoubleTapListener(this);
		gestureDetector.setIsLongpressEnabled(true);
	}

	public void setMaster(BoilerPlate master){
		Master = master;
	}


	/**
	 * GESTURES
	 */

	private static final int SWIPE_MIN_DISTANCE = 250;
	private static final int SWIPE_MAX_OFF_PATH = 200;
	private static final int SWIPE_THRESHOLD_VELOCITY = 200;

	class mGestureDetector extends GestureDetector.SimpleOnGestureListener {

		@Override
		public boolean onDown(MotionEvent e) {
			return false;
		}

		@Override
		public void onLongPress(MotionEvent e) {
			// empty
		}

		@Override
		public boolean onScroll(MotionEvent e1, MotionEvent e2,
				float distanceX, float distanceY) {
			return super.onScroll(e1, e2, distanceX, distanceY);
		}

		@Override
		public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX,
				float velocityY) {
			String msg = "";
			try {
				if (Math.abs(e1.getY() - e2.getY()) > SWIPE_MAX_OFF_PATH)
					return false;
				// right to left swipe
				if (((e1.getX() - e2.getX()) > SWIPE_MIN_DISTANCE)
						&& (Math.abs(velocityX) > SWIPE_THRESHOLD_VELOCITY)) {

					if (mWV.canGoForward()) {
						mWV.invalidate();
						mWV.goForward();
						msg = getResources().getString(R.string.historynext);
					}

					// left to right swipe
				} else if (((e2.getX() - e1.getX()) > SWIPE_MIN_DISTANCE)
						&& (Math.abs(velocityX) > SWIPE_THRESHOLD_VELOCITY)) {

					if (mWV.canGoBack()) {
						mWV.invalidate();
						mWV.goBack();
						msg = getResources().getString(R.string.historyprev);
					}
				}

				if (!msg.matches("")) {
					Toast toast = Toast.makeText(mContext, msg, Toast.LENGTH_SHORT);
					toast.setGravity(Gravity.TOP | Gravity.CENTER, 0, 0);
					toast.show();
				}
				// return true; <== using this won't work!
			} catch (Exception e) {
				// nothing
			}
			return false;
		}

	}

	@Override
	public boolean onTouchEvent(MotionEvent ev) {
		// fix for documented focus bug
		switch (ev.getAction()) {
		case MotionEvent.ACTION_DOWN:
		case MotionEvent.ACTION_UP:
			if (!hasFocus()) {
				requestFocus();
			}
			break;
		}

		return gestureDetector.onTouchEvent(ev) || super.onTouchEvent(ev);
	}

	@Override
	public boolean onDoubleTap(MotionEvent event) {
		// we don't want the default zoom idiocy
		return true;
	}

	@Override
	public boolean onDoubleTapEvent(MotionEvent event) {
		// Toast.makeText(mContext, "onDoubleTapEvent",
		// Toast.LENGTH_SHORT).show();
		return false;
	}

	@Override
	public boolean onSingleTapConfirmed(MotionEvent event) {
		// Toast.makeText(mContext, "onSingleTapConfirmed", Toast.LENGTH_SHORT)
		// .show();
		return false;
	}
}
