package BPpackage;
 
/*
 * WavTrack.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * R. Earle Harris (evsiz@this-voice.org) copyrighted 2014
 *
 */

 
/*
 * WavTrack.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * R. Earle Harris (evsiz@this-voice.org) copyrighted 2014
 *
 */


import android.content.Context;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.os.Environment;

import java.io.InputStream;
import java.io.IOException;
import java.lang.IllegalStateException;
import java.lang.InterruptedException;
import java.lang.Thread;


public class WavTrack {

	/**

	The instantiating class is responsible for everything here, including
	calling finish() before setting the instance to null; The gc call is
	helpful because the tracks slowly load up over time and normal gc is
	a long wait for a train don't come.

    Make sure you set the sample rate, stereo/mono and encoding correctly
    for your audio files.	Make sure the instantiating class has
    a public void trackEnded() for the callback. Here is mine:

	public void trackEnded(){
		if (repeating){
			newTrack();
		} else {
			paused = true;
			playing = false;
			mTrack.pause();
			mTrack.finished();
			mTrack = null;
		}
	}

	**/

    public AudioTrack at;
    public Context mContext;
    public InputStream is;
	public BoilerPlate parent;
    public boolean paused = false;
    public byte[] buffer = null;
    public int minBufferSize;
    public int position = 0;
    public int size = 0;
	public int sampleRate = 16000;
	public float factor;

	public WavTrack(Context context, int track, float speed) {
        mContext = context;
		parent = (BoilerPlate) context;
		factor = speed;
        minBufferSize = AudioTrack.getMinBufferSize(sampleRate,
            AudioFormat.CHANNEL_CONFIGURATION_MONO, AudioFormat.ENCODING_PCM_16BIT);
        at = new AudioTrack(AudioManager.STREAM_MUSIC, sampleRate,
            AudioFormat.CHANNEL_CONFIGURATION_MONO, AudioFormat.ENCODING_PCM_16BIT,
            minBufferSize, AudioTrack.MODE_STREAM);
        is = mContext.getResources().openRawResource(track);
		int playbackRate = (int) ( (float) sampleRate * factor );
		at.setPlaybackRate(playbackRate);

        try{
            size = (int)is.available();
        } catch (IOException e) {
            e.printStackTrace();
        }

        at.play();
		at.setStereoVolume (0.0f, 0.0f);

        new Thread (new Runnable(){
            public void run() {
				try{
					Thread.sleep(500);
				} catch (InterruptedException ie) { ; }
				at.setStereoVolume (1.0f, 1.0f);
			}
        }).start();

        new Thread (new Runnable(){
            public void run() {
                int i = 0;
                try{
                    buffer = new byte[512];
                    while(((i = is.read(buffer)) != -1) && !paused){
                        at.write(buffer, 0, i);
                        position += i;
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
				if (!paused){
					parent.trackEnded();
				}
            }
        }).start();
	}

	public void changeSpeed(float factor){
		int playbackRate = (int) ( (float) sampleRate * factor );
		at.setPlaybackRate(playbackRate);
	}

	public void finished(){
		try {
			at.stop();
			at.release();
		} catch (IllegalStateException ile) { ; }
		System.gc();
	}

    public void pause(){
        paused = true;
        at.pause();
        is.mark(position);
    }

    public void play() {
        paused = false;
        try {
            is.reset();
        } catch (IOException e) {
            e.printStackTrace();
        }
        at.play();

        new Thread (new Runnable(){
            public void run() {
                int i = 0;
                try{
                    buffer = new byte[512];
                    while(((i = is.read(buffer)) != -1) && !paused){
                        at.write(buffer, 0, i);
                        position += i;
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
				if (!paused){
					parent.trackEnded();
				}
            }
        }).start();
    }
}
