package BPpackage;
 
/*
 * CardSelection.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * R. Earle Harris (evsiz@this-voice.org) copyrighted 2014
 *
 */


import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.text.InputType;
import android.view.View;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import java.util.ArrayList;
import java.util.List;

public class CardSelection extends ListActivity {

	public String[] allCards;
	public String deck;
	public String CARDS = "Cards";
	public int count; 
    public int total;
    
	public PRAFileHandler fhandler = new PRAFileHandler(this);

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Intent origin = this.getIntent();
		deck = origin.getStringExtra("DECK");


		if (deck.matches("last_test.txt")){
			String[] selection = fhandler.getFileAsArray("last_test.txt");
			String[] headers = fhandler.getFileAsArray("last_hdrs.txt");
			Intent intent = new Intent();
			intent.putExtra("SELECTION", selection);
			intent.putExtra("HEADERS", headers[0]);
			intent.putExtra("SIZES", headers[1]);
			setResult(RESULT_OK, intent);
			finish();
		} else {

		String appname = getResources().getString(R.string.app_name);
		if (appname.matches("Flashcard Mojo")){
				allCards = fhandler.getExtFileAsArray("", deck);
			} else {
				allCards = fhandler.getAssetAsArray(deck);
			}
			total = allCards.length; // size of deck
			total = total - 2; //category and size header line

			AlertDialog.Builder alert = new AlertDialog.Builder(this);
			
			alert.setTitle("Set Subdeck Size");
			alert.setMessage("Total cards in deck: " + Integer.toString(total));
			final EditText input = new EditText(this);
			input.setInputType(InputType.TYPE_CLASS_NUMBER);
			alert.setView(input);

			alert.setPositiveButton("Set", new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						try {
							count = Integer.parseInt(input.getText().toString());
							setDeck(count);
						} catch (NumberFormatException nfe) {
							setDeck(total);
						}
					}
				});

			alert.setNegativeButton("Cancel",
					new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog, int whichButton) {
							finish();
						}
					});
			alert.show();
		}
		
	}
	
	private void setDeck(int count){
		
		if (count < 2){ count = 2; }
		if (count > total){ count = total; }
		int chunks = total / count;
		int remainder = total % count;

		List<String> tmp = new ArrayList<String>();

		for (int i = 0; i < chunks; i++) {
			String tmpStr = CARDS + " " + Integer.toString((i * count) + 1)
					+ " - " + Integer.toString((i + 1) * count);
			tmp.add(tmpStr);
		}

		int begin;
		int end;
		if (remainder != 0) {
			begin = ((chunks * count) + 1);
			end = begin + remainder - 1;
			String tmpS = CARDS + " " + Integer.toString(begin) + " - "
					+ Integer.toString(end);
			tmp.add(tmpS);
		}

		String[] tmpA = new String[tmp.size()];
		tmp.toArray(tmpA);
		final String[] SUBDECKS = tmpA;
		setContentView(R.layout.activity_card_selection);
		TextView title = (TextView) findViewById(R.id.title);
		setListAdapter(new ArrayAdapter<String>(this,
				android.R.layout.simple_list_item_1, SUBDECKS));

		EdgeEffectListView listView = (EdgeEffectListView) getListView();
		listView.setTextFilterEnabled(true);

		listView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
				String result = ((TextView) view).getText().toString();
				String[] tokens = result.split(" ");
				int a = Integer.parseInt(tokens[1]);
				int b = Integer.parseInt(tokens[3]);
				int size = (b - a) + 1;
				String[] selection = new String[size];
				for (int i = 0; i < size; i++) {
					selection[i] = allCards[(i+2) + a - 1];// + for header lines
				}

				Intent intent = new Intent();
				intent.putExtra("SELECTION", selection);
				intent.putExtra("SET", result);
				setResult(RESULT_OK, intent);
				finish();
			}
		});
	}

	@Override
	public void onBackPressed() {
		Intent intent = new Intent();
		setResult(RESULT_CANCELED, intent);
		finish();
	}

}
