package BPpackage;
 
/*
 * BatteryService.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * R. Earle Harris (evsiz@this-voice.org) copyrighted 2014
 *
 */


import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.preference.PreferenceManager;
import android.support.v4.app.NotificationCompat;

public class BatteryService extends Service {
	 
	 /**
	  * Notifications
	  */
	private static final int lightIcon0 = R.drawable.light000;
	private static final int darkIcon0  = R.drawable.dark000;
	private NotificationManager mNMgr;

	public void notifier(int id, String msg, int pct){
		Intent notificationIntent = new Intent(this, BoilerPlate.class);
		PendingIntent mPendingIntent = PendingIntent.getActivity(this, 0, notificationIntent, 0);
		NotificationCompat.Builder notebld = new NotificationCompat.Builder(this)
			.setContentTitle("BoilerPlate")
			.setContentText(msg)
			.setContentIntent(mPendingIntent);
			
		if (pct == -1) {
			notebld.setSmallIcon(R.drawable.icon);
		} else {
			if (whiteNumbers){
				notebld.setSmallIcon(lightIcon0 + pct);
			} else {
				notebld.setSmallIcon(darkIcon0 + pct);
			}
		}
		Notification note = notebld.build();
		if (id == 1){
			note.flags |= Notification.FLAG_ONGOING_EVENT | Notification.FLAG_NO_CLEAR;
		}
		mNMgr.notify(id, note);
	 }

	/**
	 * Messenger
	 */
	final Messenger mMessenger = new Messenger(new IncomingHandler());

	static final int MSG_NUMS_00 = 1;
	static final int MSG_NUMS_FF = 2;
	static final int MSG_QCHK_NO = 3;
	static final int MSG_QCHK_OK = 4;

	static boolean qualityCheck = true;
	static boolean whiteNumbers = true;
	static int last_percent = 0;
	
	static class IncomingHandler extends Handler {
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case MSG_NUMS_00:
				whiteNumbers = false;
				last_percent = 0;
				break;
			case MSG_NUMS_FF:
				whiteNumbers = true;
				last_percent = 0;
				break;
			case MSG_QCHK_NO:
				qualityCheck = false;
				break;
			case MSG_QCHK_OK:
				qualityCheck = true;
				break;
			}
		}
	}
	
	/**
	 * BroadcastReceiver
	 */ 
    private final IntentFilter batteryChanged = new IntentFilter(Intent.ACTION_BATTERY_CHANGED);

    public static final int HEALTH_UNKNOWN     = 1;
    public static final int HEALTH_OVERHEAT    = 3;
    public static final int HEALTH_DEAD        = 4;
    public static final int HEALTH_OVERVOLTAGE = 5;
    public static final int HEALTH_FAILURE     = 6;
	
    
	private final BroadcastReceiver mBatteryReceiver = new BroadcastReceiver() {
	        
		@Override
	    public void onReceive(Context context, Intent intent) { 
			
			// battery percentage
            int level = intent.getIntExtra("level", -1);
            int scale = intent.getIntExtra("scale", -1);
            int pct = level * 100 / scale;
            if (pct != last_percent){
            	last_percent = pct;
                Integer percent = pct; 
                String msg = percent.toString() + "% battery";
                notifier(1, msg, pct);
            }
            
            // battery health
            int health = intent.getIntExtra("health", -1);
            String status = "Battery ";
            switch(health){
            case HEALTH_UNKNOWN:
            	status += "health unknown.";
            	break;
            case HEALTH_OVERHEAT:
            	status += "overheating.";
            	break;
            case HEALTH_DEAD:
            	status += "dead.";
            	break;
            case HEALTH_OVERVOLTAGE:
            	status += "over voltage.";
            	break;
            case HEALTH_FAILURE:
            	status += "failing.";
            	break;
            }
            if (qualityCheck && !status.matches("Battery ")){
            	notifier(2, status, -1);
            }            
	     }
	 };
	
	/**
	 * Android States
	 */
	
	@Override
	public IBinder onBind(Intent intent) {
		return mMessenger.getBinder();
	}

	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		return START_STICKY;
	}

	@Override
	public void onCreate() {
		super.onCreate();
		registerReceiver(mBatteryReceiver, batteryChanged);
		mNMgr = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
		SharedPreferences prefs = PreferenceManager
				.getDefaultSharedPreferences(getBaseContext());

		boolean state = prefs.getBoolean("numbers_checkbox", false);
		if (state) {
			whiteNumbers = false;
		} else {
			whiteNumbers = true;
		}
		state = prefs.getBoolean("quality_checkbox", false);
		if (state) {
			qualityCheck = false;
		} else {
			qualityCheck = true;
		}
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		mNMgr.cancelAll();
		unregisterReceiver(mBatteryReceiver);
	}
}
