package BPpackage;
 
/*
 * BoilerPlate.java is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * R. Earle Harris (evsiz@this-voice.org) copyrighted 2014
 *
 */


import android.content.ComponentName;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceManager;
import android.view.View;
import android.widget.ToggleButton;

public class BoilerPlate extends PreferenceActivity implements
		OnSharedPreferenceChangeListener {

	Messenger mService = null;
	Message msg = null;
	boolean mBound = false;

	public void onSharedPreferenceChanged(SharedPreferences prefs, String key) {
		if (!mBound) { // we restart, ignore change
			connect();
			ToggleButton toggle = (ToggleButton) findViewById(R.id.toggle);
			if (toggle.isChecked()) {
				toggle.setChecked(false);
			} else {
				toggle.setChecked(true);
			}
			return;
		}

		boolean state = prefs.getBoolean(key, false);

		if (key.matches("data_checkbox")) {
			if (state) {
				transmit(NetworkService.MSG_DATA_1);
			} else {
				transmit(NetworkService.MSG_DATA_0);
			}
		} else if (key.matches("wifi_checkbox")) {
			if (state) {
				transmit(NetworkService.MSG_WIFI_1);
			} else {
				transmit(NetworkService.MSG_WIFI_0);
			}
		}
	}

	private void transmit(int i) {
		msg = Message.obtain(null, i, 0, 0);
		try {
			mService.send(msg);
		} catch (RemoteException e) {
			;// TODO
		}
	}

	/** Defines callbacks for service binding, passed to bindService() */
	private ServiceConnection mConnection = new ServiceConnection() {

		@Override
		public void onServiceConnected(ComponentName className, IBinder service) {
			mService = new Messenger(service);
			mBound = true;
		}

		@Override
		public void onServiceDisconnected(ComponentName arg0) {
			mBound = false;
		}
	};

	/**
	 * PreferenceChangeListener methods
	 */

	private static Preference.OnPreferenceChangeListener sBindPreferenceSummaryToValueListener = new Preference.OnPreferenceChangeListener() {
		@Override
		public boolean onPreferenceChange(Preference preference, Object value) {
			String stringValue = value.toString();

			if (preference instanceof ListPreference) {
				// For list, look up correct value in list's 'entries' list.
				ListPreference listPreference = (ListPreference) preference;
				int index = listPreference.findIndexOfValue(stringValue);
				// Set the summary to reflect the new value.
				preference
						.setSummary(index >= 0 ? listPreference.getEntries()[index]
								: null);
			} else {
				// else simple string
				preference.setSummary(stringValue);
			}
			return true;
		}
	};

	private static void bindPreferenceSummaryToValue(Preference preference) {
		// watch for value changes.
		preference
				.setOnPreferenceChangeListener(sBindPreferenceSummaryToValueListener);
		// Trigger immediately with preference's current value.
		sBindPreferenceSummaryToValueListener.onPreferenceChange(
				preference,
				PreferenceManager.getDefaultSharedPreferences(
						preference.getContext()).getString(preference.getKey(),
						""));
	}

	/**
	 * UI Button methods
	 */

	public void onOff(View view) {
		if (mBound) {
			try {
				unbindService(mConnection);
				mBound = false;
				stopService(new Intent(this, NetworkService.class));
				mService = null;
			} catch (Exception e) {
				;// TODO
			}
		} else {
			connect();
		}
	}

	public void webHelp(View view) {
		String url = "http://YOUR_SITE/boilerplate.html";
		Intent i = new Intent(Intent.ACTION_VIEW);
		i.setData(Uri.parse(url));
		startActivity(i);
	}

	public void emailDeveloper(View view) {
		Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND);
		String[] recipients = new String[] { "YOUR_EMAIL",
				"", };
		emailIntent.putExtra(android.content.Intent.EXTRA_EMAIL, recipients);
		emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT,
				"BoilerPlate");
		emailIntent.putExtra(android.content.Intent.EXTRA_TEXT,
				"\n\n\n\n\n\n\n\nI\'m on a " + Build.MANUFACTURER + " "
						+ Build.MODEL + " using Android "
						+ Build.VERSION.RELEASE);
		emailIntent.setType("text/plain");
		startActivity(Intent
				.createChooser(emailIntent, "Email BoilerPlate..."));
	}

	/**
	 * Android state methods
	 */

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		final SharedPreferences preferences = PreferenceManager
				.getDefaultSharedPreferences(getBaseContext());
		preferences.registerOnSharedPreferenceChangeListener(this);
		setContentView(R.layout.activity_boilerplate);
		new SimpleEula(this).show();
	}

	@Override
	protected void onPostCreate(Bundle savedInstanceState) {
		super.onPostCreate(savedInstanceState);
		addPreferencesFromResource(R.xml.pref_general);
	}

	@Override
	protected void onStart() {
		super.onStart();
		connect();
	}

	private void connect() {
		Intent intent = new Intent(this, NetworkService.class);
		startService(intent);
		bindService(intent, mConnection, 0);
	}

	@Override
	protected void onPause() {
		super.onPause();
		getPreferenceScreen().getSharedPreferences()
				.unregisterOnSharedPreferenceChangeListener(this);
	}

	@Override
	protected void onResume() {
		super.onResume();
		getPreferenceScreen().getSharedPreferences()
				.registerOnSharedPreferenceChangeListener(this);
	}

	@Override
	protected void onStop() {
		super.onStop();
		if (mBound) { // Unbind from the service
			unbindService(mConnection);
			mBound = false;
		}
	}

}
