/*
  TimerService Client
  1. add TextView time and Buttons pause and toggle to Activity's XML
  2. create logic for loading timer
  3. put a meaningful evaluation in updateTimer
  4. put android-support-v4.jar in $PRIDE/$NAME/libs
 */


    /*
     * Timer Vars
     */
	public final int TIMER = 0;
	private TimerUpdateReceiver timerUpdateReceiver = null;
	public TimerService mTimerService = null;
    public Button pause;
    public Button toggle;
    public TextView time;
	public boolean mTimerBound = false;
	public boolean paused = false;
	public boolean started = false;
	public int savedSeconds;
	public double totalSecondsRemaining;
	public int minutesRemaining;
	public int secondsRemaining;
	public int timerSeconds;


    /*
     * Pause Button
     */
    public void pause(View view){
		paused = !paused;
		if (paused) {
			pause.setText("Resume");
			savedSeconds = (int) totalSecondsRemaining;
			if (mTimerBound) {
				mTimerService.stopCountdown();
			}
		} else {
			mTimerService.startCountdown(savedSeconds);
			pause.setText("Pause");
		}
    }


    /*
     * Start/Stop Button
     */
    public void startStop(View view){
		started = !started;
		if (started) {
			paused = false;
			toggle.setText("Stop");
			mTimerService.startCountdown(0);
		} else {
			final Dialog dialog = new Dialog(this);
			dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
			dialog.setContentView(R.layout.alert_two);
			TextView t = (TextView) dialog.findViewById(R.id.dTitle);
			t.setText("Stop Timer");
			TextView m = (TextView) dialog.findViewById(R.id.dMessage);
			m.setText("Pause would allow resume.");
			
			Button pos = (Button) dialog.findViewById(R.id.dPos);
			pos.setText("Stop");
			pos.setOnClickListener(new OnClickListener() {
					@Override
						public void onClick(View v) {
						stop();
						dialog.dismiss();
						}
				});
			
			Button neg = (Button) dialog.findViewById(R.id.dNeg);
			neg.setText("Cancel");
			neg.setOnClickListener(new OnClickListener() {
					@Override
						public void onClick(View v) {
						started = !started;
						dialog.dismiss();
					}
				});

			dialog.show();
		}
	}

    // called from stop dialog
	public void stop(){
		if (mTimerBound) {
			mTimerService.stopCountdown();
		}
		toggle.setText("Start");
		loadTimer();
	}



    // called from service
	public void updateTimer(long timeRemaining){
		if (paused){ return; }
		totalSecondsRemaining = timeRemaining / 1000.0;
		minutesRemaining = (int) totalSecondsRemaining / 60;
		secondsRemaining = (int) totalSecondsRemaining - (minutesRemaining * 60);
		if (/* timer is in meaningful state*/){
			//do something
		}
		String minutes = new Integer(minutesRemaining).toString();
		String seconds = new Integer(secondsRemaining).toString();
		time.setText(String.format("%d:%02d", minutesRemaining, secondsRemaining));
	}

	@Override
    protected void onStart() {
        super.onStart(); 
		Intent intent = new Intent(this, TimerService.class);
		bindService(intent, mConnection, Context.BIND_AUTO_CREATE);
    }
	
    @Override
    protected void onPause() {
        super.onPause();
		savedSeconds = (int) totalSecondsRemaining;
		paused = true;
		pause.setText("Resume");
		if (mTimerBound) {
			mTimerService.stopCountdown();
		}
        if (timerUpdateReceiver != null) {
        	LocalBroadcastManager.getInstance(this).unregisterReceiver(timerUpdateReceiver);
        	timerUpdateReceiver = null;
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
		if (!mTimerBound){
			Intent intent = new Intent(this, TimerService.class);
			bindService(intent, mConnection, Context.BIND_AUTO_CREATE);
		}
        if (timerUpdateReceiver == null) {
        	timerUpdateReceiver = new TimerUpdateReceiver();
        }
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(TimerService.UPDATE_TIMER_INTENT);
        intentFilter.addAction(TimerService.TIMER_FINISHED_INTENT);
        LocalBroadcastManager.getInstance(this).registerReceiver(timerUpdateReceiver, intentFilter);
    }

	@Override
    protected void onStop() {
        super.onStop();
        if (mTimerBound) {
            unbindService(mConnection);
            mTimerBound = false;
        }
    }


	private class TimerUpdateReceiver extends BroadcastReceiver {

	    @Override
	    public void onReceive(Context context, Intent intent) {
	        if (intent.getAction().equals(TimerService.UPDATE_TIMER_INTENT)) {
	        	updateTimer(intent.getExtras().getLong("timeRemainingMs"));

	        } else if (intent.getAction().equals(TimerService.TIMER_FINISHED_INTENT)) { 
	        	switch (currentState) {
					started = !started;
					toggle.setText("Start");
	        		// load timer again
	        	}
	        }
	    }
	}

	
	private ServiceConnection mConnection = new ServiceConnection() {
		
		@Override
	    public void onServiceConnected(ComponentName className, IBinder service) {
	        mTimerService = ((TimerService.LocalBinder) service).getService();
	        mTimerBound = true;
	    }

		@Override
	    public void onServiceDisconnected(ComponentName className) {
	    	mTimerBound = false;
	    }
	};

